var connect = require("connect");
var http = require('http');
var url = require('url');

var faye = require('faye');
var bayeux = new faye.NodeAdapter({mount: '/faye', timeout: 45});

var devices = [];

//HTTP server for testing, not necessary for final implementation
//if there are going to be clients pushing to the server updates
var app = connect()
  .use(connect.logger('dev'))
  .use('/update', function(req, res){
		//Parse URL
		update = url.parse(req.originalUrl, true);

		var device_list = [];
		if ("device" in update.query)
		{
			//Send response
			device_list.push(update.query["device"]);
		}
		else
		{
			device_list = devices;
		}

		if (update.query["color"])
		{
			//Send response
			res.end("Publishing color #" + update.query["color"] + " to devices " + device_list + '\n');

			for(var i = device_list.length - 1; i >= 0; i--) {
				//Publish to server
				bayeux.getClient().publish('/devices/' + device_list[i], 
					{ 
						color: "#" + update.query["color"],
						channel: update.query["channel"],
						firstLed: update.query["firstLed"],
						lastLed: update.query["lastLed"]
					});
			}
		}
		else if (update.query["status"])
		{
			res.end("Publishing status " + update.query["status"] + " to devices " + device_list + '\n');
			
			for(var i = device_list.length - 1; i >= 0; i--) {
				//Publish to server
				bayeux.getClient().publish('/devices/' + device_list[i], 
					{ 
						status: update.query["status"]
					});
			}

		}
		else if (update.query["pattern"])
		{
			//Send response
			res.end("Publishing pattern " + update.query["pattern"] + " to devices " + device_list + '\n');

			for(var i = device_list.length - 1; i >= 0; i--) {
				//Publish to server
				bayeux.getClient().publish('/devices/' + device_list[i], 
						{ 
							pattern: update.query["pattern"], 
							channel: update.query["channel"],
							firstLed: update.query["firstLed"],
							lastLed: update.query["lastLed"],
							repeat: update.query["repeat"],
							duration: update.query["duration"]
						});
			}
		}
		else 
		{
			res.end("Unable to process request...");
		}

  });

server = http.createServer(app);

//Push server debug logging
bayeux
	.on('handshake', function(clientId) {
		console.log('[FAYE] handshake %s', clientId);
	})
	.on('subscribe', function(clientId, channel) {
		console.log('[FAYE] subscribe %s %s', clientId, channel);

		device_id = channel.match(/^\/devices\/(.*)$/);
		if (device_id)
		{
			devices.push(device_id[1]);
		}

		console.log(devices);
	})
	.on('unsubscribe', function(clientId, channel) {
		console.log('[FAYE] unsubscribe %s %s', clientId, channel);

		device_id = channel.match(/^\/devices\/(.*)$/);
		if (device_id)
		{
			var index = devices.indexOf(device_id[1]);
			if (index > -1)
			{
				devices.splice(index, 1);
			}
		}

		console.log(devices);
	})
	.on('publish', function(clientId, channel, data) {
		console.log('[FAYE] publish %s %s', clientId, channel, data);
	})
	.on('disconnect', function(clientId) {
		console.log('[FAYE] disconnect %s', clientId);
	})


//Attach push server to the same HTTP server
bayeux.attach(server);

//Start listening for connections
server.listen(3000);
